//////////////////////////////////////////////////////////////////////////
//
// Hall A analyzer replay script
//
// if there is no rootlogon.C, execute following command in analyzer first
// gSystem->Load("<your lib name>.so");
// gSystem->AddIncludePath("-I<your include path>");
// gInterpreter->AddIncludePath("<your include path>");
//
//////////////////////////////////////////////////////////////////////////
//	
//	Author : Jin Huang (jinhuang@jlab.org)    Aug 2007
//      also refered to older version
//
//	Modify History:
//      Mar 2008    Jin Huang
//          add physics module support
//      Mar 2008    Jin Huang
//          make it smarter
//
//////////////////////////////////////////////////////////////////////////

#include "def.h"

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <fstream>
#include <string>
#include <string.h>

#include "TSystem.h"
#include "TROOT.h"
#include "THaEvent.h"
#include "THaRun.h"
#include "THaAnalyzer.h"
#include "THaTriggerPlane.h"
#include "THaScintPlane.h"

#include "THaBigBiteTpMwdc.h"
#include "THaTriggerPlane.h"
#include "THaScintPlane.h"
#include "MWDC.h"
#include "THaOpticsAnalytical.h"

#include "THaHRS.h"
#include "THaUnRasteredBeam.h"
#include "THaRasteredBeam.h"
#include "THaDecData.h"
#include "THaCherenkov.h"
#include "THaShower.h"
#include "THaGoldenTrack.h"
#include "THaReactionPoint.h"
#include "THaExtTarCor.h"
#include "THaScalerGroup.h"
#include "THaPrimaryKine.h"
#include "THaSecondaryKine.h"
#include "THaIdealBeam.h"

using namespace std;



void replay_phys(Int_t runnumber=0,Int_t all=0)
{
    //general replay script
    //for detector or physics replay, please use replay_det or physics reply 

    cout<<"replay: Init analyzer ..."<<endl;
    THaAnalyzer* analyzer = THaAnalyzer::GetInstance();
    if( analyzer ) {
        analyzer->Close();
    } else {
        analyzer = new THaAnalyzer;
    }



    // step 1: add detectors
    THaBigBite* pB=0;

    cout<<"replay: Adding BigBite ..."<<endl;
    pB=new THaBigBite("BB","BigBite"); 
    gHaApps->Add(pB);

    pB->AddDetector(new THaTriggerPlane("tp","Trigger Plane",pB));

    pB->AddDetector(new TreeSearch::MWDC("mwdc","MWDC",pB));

    pB->MountOptics(new THaOpticsAnalytical("optics","BigBite Optics Child Class",pB,"Beam"));

    cout<<"replay: Adding Beam ..."<<endl;
    gHaApps->Add(new THaIdealBeam("Beam","Idea Beam, for Test Only"));
    gHaApps->Add(new THaUnRasteredBeam("urb","Unrastered beam"));
    gHaApps->Add(new THaRasteredBeam("rb","Rastered Beam"));

    gHaApps->Add(new THaDecData("DL","Misc. Decoder Data"));

    cout<<"replay: Adding HRS-L ..."<<endl;
    THaApparatus* HRSL = new THaHRS("L","Left HRS");
    gHaApps->Add( HRSL );

    // add detectors that are not in the default config
    HRSL->AddDetector( new THaCherenkov("cer", "Gas Cherenkov counter" ));
    HRSL->AddDetector( new THaShower("prl1", "Pre-shower pion rej." ));
    HRSL->AddDetector( new THaShower("prl2", "Show pion rej." ));


    //cout<<"replay: Adding HRS-R ..."<<endl;
    //THaApparatus* HRSR = new THaHRS("R","Right HRS");
    //gHaApps->Add( HRSR );

    // add detectors that are not in the default config
    //HRSR->AddDetector( new THaCherenkov("cer", "Gas Cherenkov counter" ));
    //HRSR->AddDetector( new THaShower("ps", "Pre-shower" ));
    //HRSR->AddDetector( new THaShower("sh", "Shower" ));


    //Step 1.5: load Physics modules
    cout<<"replay: adding Physics modules ..."<<endl;

    gHaPhysics->Add( new THaGoldenTrack("L.gold","Golden track for LHRS", "L") );

    THaPhysicsModule *Rpt_l = new THaReactionPoint(
        "ReactPt_L","Reaction vertex for Left","L","Beam");
    gHaPhysics->Add( Rpt_l );

    // Correct for using an Extended target
    // This needs information about the Point of interaction 
    // (hence a THaVertexModule)
    THaPhysicsModule* TgC_l = new THaExtTarCor("ExTgtCor_L",
        "Corrected for extended target, HRS-L",
        "L","ReactPt_L");
    gHaPhysics->Add( TgC_l );

    //The CORRECTED Electron kinematics
    THaPrimaryKine *PriKine=new THaPrimaryKine(
        "PriKine","kinematics of scattering of the primary (beam) particle",
        "ExTgtCor_L","Beam",.938272029);
    gHaApps->Add(PriKine);

    THaSecondaryKine *SecKine=new THaSecondaryKine(
        "SecKine","kinematics of scattering of the primary (beam) particle",
        "BB","PriKine",.938272029);
    gHaApps->Add(SecKine);

    // add scalers
    //THaScalerGroup* LeftScalers = new THaScalerGroup("Left");
    //gHaScalers->Add(LeftScalers);
    //// Enable scalers
    //analyzer->EnableScalers();




    // step 2: setup run information

    int nrun, nev;
    int found = 0;
    const char** path = 0;
    char filename[300],buf[300];  
    FILE *fd;

    while( found==0 ) {
        if (runnumber<=0)
        {
            cout << "\nreplay: Please enter a Run Number (-1 to exit):";
            cin >> nrun;
            if( nrun<=0 ) break;
        }
        else 
            nrun=runnumber;

        path=PATHS;

        while ( path && *path ) {
            sprintf(filename,RAW_DATA_FORMAT,*path,nrun);
            cout<<"replay: Try file "<<filename<<endl;
            fd = fopen(filename,"r");
            if (fd != NULL) {
                found = 1;
                cout << "replay: Will analyze file "<<filename<<endl;
                fclose(fd);
                break;
            }
            path++;
        }
        if ( (!path || !*path) && !found ) {
            cout << "replay: File not found. \n";
            if (runnumber>0) runnumber=0;
        }
    }
    if(nrun<=0) return;

    if (all==0)
    {
        cout << "\nreplay: Number of Events to analyze "
            <<" (default=";
        if (DEF_REPLAY_NUM>0)
            cout<<DEF_REPLAY_NUM<<"):";
        else
            cout<<"replay all):";

        //trick to supports blank inputs
        fgets(buf,300,stdin);
        if (sscanf(buf,"%d\n",&nev)<1)
        {
            fgets(buf,300,stdin);
            if (sscanf(buf,"%d\n",&nev)<1)
                nev=DEF_REPLAY_NUM;   
        }
    }
    else 
        nev=all;

    cout<<"replay: Setup run inputs/outputs ..."<<endl;

    THaRun run(filename);
    run.SetFirstEvent( 1 );
    if(nev>-1) run.SetLastEvent(nev);

    char sumname[300];
    char outname[300];
    sprintf(outname,OUT_FILE_FORMAT,nrun);
    analyzer->SetOutFile( outname );
    analyzer->SetOdefFile(OUT_DEFINE_FILE);
    analyzer->SetCutFile(CUT_DEFINE_FILE);  
    sprintf(sumname,SUMMARY_PHYSICS_FORMAT,nrun);
    analyzer->SetSummaryFile(sumname); // optional
    analyzer->SetEvent( new THaEvent );



    // step 3: run it
    cout<<"replay: Start Process ..."<<endl;
    analyzer->Process(run);     // start the actual analysis



    // step 3: clean up
    run.Close();
    gHaApps->Delete();
    analyzer->Close();

    cout<<"replay: YOU JUST ANALYZED RUN number "<<nrun<<"."<<endl;
}
